==> trend_etl-1.0.1.tar.gz: trend_etl-1.0.1/trend_etl/transform/submissions.py <==
from pyspark.sql import DataFrame

from pyspark.sql.functions import (
    col,
    lit,
    to_json,
    named_struct,
    format_string,
)
from pyspark.sql.types import StringType, BinaryType, TimestampType, LongType

import trend_etl.schema.fields as fields
import trend_etl.schema.args as arguments
from trend_etl.functions.index import (
    jobid_2_timestamp,
    parse_reddit_submissions,
    parse_pushshift_submissions,
    get_created_day,
)


def from_reddit_archive_get_bronze_submissions_df(archive_df: DataFrame) -> DataFrame:
    parsed = archive_df.withColumn(
        fields.archive_value_field,
        parse_reddit_submissions(col(fields.archive_value_field)),
    )
    created_utc_field = col(f"{fields.archive_value_field}.data.created_utc")

    return parsed.select(
        to_json(col(fields.archive_value_field))
        .cast(BinaryType())
        .alias(fields.bronze_value_field),
        lit("archive/reddit/submissions").alias(fields.bronze_provenance_field),
        jobid_2_timestamp(fields.archive_job_id_field).alias(
            fields.bronze_timestamp_field
        ),
        get_created_day(created_utc_field.cast(TimestampType())).alias(
            fields.created_day_field
        ),
    )


def from_pushshift_archive_get_bronze_submissions_df(
    archive_df: DataFrame,
) -> DataFrame:
    parsed = archive_df.withColumn(
        fields.archive_value_field,
        parse_pushshift_submissions(col(fields.archive_value_field)),
    )
    id_field = col(f"{fields.archive_value_field}.id")
    created_utc_field = col(f"{fields.archive_value_field}.created_utc")

    return (
        parsed.withColumn(
            fields.archive_value_field,
            col(fields.archive_value_field).withField(
                "name",
                format_string("t3_%s", id_field),
            ),
        )
        .withColumn(
            fields.archive_value_field,
            col(fields.archive_value_field).withField(
                "created_utc",
                created_utc_field.cast(LongType()),
            ),
        )
        .select(
            to_json(
                named_struct(
                    lit("kind"),
                    lit("t3"),
                    lit("data"),
                    col(fields.archive_value_field),
                    lit("note"),
                    lit("pushshift_archive"),
                )
            )
            .cast(BinaryType())
            .alias(fields.bronze_value_field),
            lit("archive/pushshift/submissions").alias(fields.bronze_provenance_field),
            jobid_2_timestamp(fields.archive_job_id_field).alias(
                fields.bronze_timestamp_field
            ),
            get_created_day(created_utc_field.cast(TimestampType())).alias(
                fields.created_day_field
            ),
        )
    )


# arguments:
#   input_df: dataframe received from reading a kafka topic
#   target_topic: source topic from kafka
# return: a dataframe in the schema expected for bronze tables (for all reddit data kinds)
def from_kafka_get_bronze_submissions_df(input_df: DataFrame, target_topic: str):
    parsed = input_df.withColumn(
        fields.kafka_value_field,
        parse_reddit_submissions(col(fields.kafka_value_field).cast(StringType())),
    )
    created_utc_field = col(f"{fields.archive_value_field}.data.created_utc")

    return parsed.select(
        to_json(col(fields.kafka_value_field))
        .cast(BinaryType())
        .alias(fields.bronze_value_field),
        lit(f"kafka/{target_topic}").alias(fields.bronze_provenance_field),
        col(fields.kafka_timestamp_field).alias(fields.bronze_timestamp_field),
        get_created_day(created_utc_field.cast(TimestampType())).alias(
            fields.created_day_field
        ),
    )


# arguments:
#   streaming_bronze: streaming dataframe from bronze table
#   subreddit_idents: subreddit uuid idents table
#   submission_idents: submission uuid idents table
#   bot_authors: table of bot authors
# return: dataframe in the schema of `reddit_submissions`
def from_bronze_get_submissions_df(
    streaming_bronze: DataFrame,
    subreddit_idents: DataFrame,
    submission_idents: DataFrame,
    bot_authors: DataFrame,
) -> DataFrame:
    parsed_df = streaming_bronze.select(
        parse_reddit_submissions(
            col(fields.bronze_value_field).cast(StringType())
        ).alias("value"),
        col(fields.bronze_timestamp_field),
    )
    flattened_df = parsed_df.select(
        col("value.data.*"),
        col(fields.bronze_timestamp_field),
    )

    # filter out bot submissions
    bot_filtered_df = flattened_df.join(
        bot_authors, fields.reddit_bot_users_author, "left_anti"
    )

    test_filtered_df = bot_filtered_df.where(
        col("subreddit_id").isin(arguments.test_subreddits) == False
    )

    # NOTE: left outer joins (below) will cause any nulls to be
    #       detected as a downstream error.

    # join with subreddit_idents
    subreddits_df = test_filtered_df.join(
        subreddit_idents.withColumnRenamed(
            fields.idents_source_id_field, "subreddit_id"
        ),
        "subreddit_id",
        "inner",  # filter out submissions from subreddit_type = 'user' and like which we don't track
    ).drop("subreddit_id")

    # join with submission_idents
    submissions_df = (
        subreddits_df.join(
            submission_idents.withColumnRenamed(fields.idents_source_id_field, "name"),
            "name",
            "left_outer",
        )
        .withColumn(fields.source_keys_field, named_struct(lit("name"), col("name")))
        .drop("name")
    )

    return (
        submissions_df.withColumn(
            fields.reddit_submissions_created_at,
            col("created_utc").cast(TimestampType()),
        )
        .drop("created_utc")
        .withColumnRenamed("selftext", fields.reddit_submissions_text)
    )

==> trend_etl-1.4.7.tar.gz: trend_etl-1.4.7/trend_etl/transform/submissions.py <==
from pyspark.sql import DataFrame

from pyspark.sql.functions import col, lit, to_json, struct, format_string, when
from pyspark.sql.types import (
    StringType,
    BinaryType,
    TimestampType,
    LongType,
    DoubleType,
)

import trend_etl.schema.fields as fields
import trend_etl.schema.args as arguments
from trend_etl.functions.index import (
    jobid_2_timestamp,
    parse_reddit_submissions,
    parse_pushshift_submissions,
    get_created_day,
    is_scientific,
)


def from_reddit_archive_get_bronze_submissions_df(archive_df: DataFrame) -> DataFrame:
    parsed = archive_df.withColumn(
        fields.archive_value_field,
        parse_reddit_submissions(col(fields.archive_value_field)),
    )
    created_utc_field = col(f"{fields.archive_value_field}.data.created_utc")

    return parsed.select(
        to_json(col(fields.archive_value_field))
        .cast(BinaryType())
        .alias(fields.bronze_value_field),
        lit("archive/reddit/submissions").alias(fields.bronze_provenance_field),
        jobid_2_timestamp(fields.archive_job_id_field).alias(
            fields.bronze_timestamp_field
        ),
        get_created_day(created_utc_field.cast(TimestampType())).alias(
            fields.created_day_field
        ),
    )


def from_pushshift_archive_get_bronze_submissions_df(
    archive_df: DataFrame,
) -> DataFrame:
    parsed = archive_df.withColumn(
        fields.archive_value_field,
        parse_pushshift_submissions(col(fields.archive_value_field)),
    )
    id_field = col(f"{fields.archive_value_field}.id")
    created_utc_field = col(f"{fields.archive_value_field}.created_utc")

    return (
        parsed.withColumn(
            fields.archive_value_field,
            col(fields.archive_value_field).withField(
                "name",
                format_string("t3_%s", id_field),
            ),
        )
        .withColumn(
            fields.archive_value_field,
            col(fields.archive_value_field).withField(
                "created_utc",
                created_utc_field.cast(LongType()),
            ),
        )
        .select(
            to_json(
                struct(
                    lit("t3").alias("kind"),
                    col(fields.archive_value_field).alias("data"),
                    lit("pushshift_archive").alias("note"),
                )
            )
            .cast(BinaryType())
            .alias(fields.bronze_value_field),
            lit("archive/pushshift/submissions").alias(fields.bronze_provenance_field),
            jobid_2_timestamp(fields.archive_job_id_field).alias(
                fields.bronze_timestamp_field
            ),
            get_created_day(created_utc_field.cast(TimestampType())).alias(
                fields.created_day_field
            ),
        )
    )


# arguments:
#   input_df: dataframe received from reading a kafka topic
#   target_topic: source topic from kafka
# return: a dataframe in the schema expected for bronze tables (for all reddit data kinds)
def from_kafka_get_bronze_submissions_df(input_df: DataFrame, target_topic: str):
    parsed = input_df.withColumn(
        fields.kafka_value_field,
        parse_reddit_submissions(col(fields.kafka_value_field).cast(StringType())),
    )
    created_utc_field = col(f"{fields.archive_value_field}.data.created_utc")

    return parsed.select(
        to_json(col(fields.kafka_value_field))
        .cast(BinaryType())
        .alias(fields.bronze_value_field),
        lit(f"kafka/{target_topic}").alias(fields.bronze_provenance_field),
        col(fields.kafka_timestamp_field).alias(fields.bronze_timestamp_field),
        get_created_day(created_utc_field.cast(TimestampType())).alias(
            fields.created_day_field
        ),
    )


# arguments:
#   streaming_bronze: streaming dataframe from bronze table
#   subreddit_idents: subreddit uuid idents table
#   submission_idents: submission uuid idents table
#   bot_authors: table of bot authors
# return: dataframe in the schema of `reddit_submissions`
def from_bronze_get_submissions_df(
    streaming_bronze: DataFrame,
    subreddit_idents: DataFrame,
    submission_idents: DataFrame,
    bot_authors: DataFrame,
) -> DataFrame:
    parsed_df = streaming_bronze.select(
        parse_reddit_submissions(
            col(fields.bronze_value_field).cast(StringType())
        ).alias("value"),
        col(fields.bronze_timestamp_field),
    )
    flattened_df = parsed_df.select(
        col("value.data.*"),
        col(fields.bronze_timestamp_field),
    )

    # filter out bot submissions
    bot_filtered_df = flattened_df.join(
        bot_authors, fields.reddit_bot_users_author, "left_anti"
    )

    test_filtered_df = bot_filtered_df.where(
        col("subreddit_id").isin(arguments.test_subreddits) == False
    )

    # NOTE: left outer joins (below) will cause any nulls to be
    #       detected as a downstream error.

    # join with subreddit_idents
    subreddits_df = test_filtered_df.join(
        subreddit_idents.withColumnRenamed(
            fields.idents_source_id_field, "subreddit_id"
        ),
        "subreddit_id",
        "inner",  # filter out submissions from subreddit_type = 'user' and like which we don't track
    ).drop("subreddit_id")

    # join with submission_idents
    submissions_df = (
        subreddits_df.join(
            submission_idents.withColumnRenamed(fields.idents_source_id_field, "name"),
            "name",
            "left_outer",
        )
        .withColumn(fields.source_keys_field, struct(col("name").alias("name")))
        .drop("name")
    )

    return (
        submissions_df.withColumn(
            fields.reddit_submissions_created_at,
            when(
                is_scientific(col("created_utc")),
                col("created_utc").cast(DoubleType()).cast(LongType()),
            )
            .otherwise(col("created_utc").cast(LongType()))
            .cast(TimestampType()),
        )
        .drop("created_utc")
        .withColumnRenamed("selftext", fields.reddit_submissions_text)
    )
